@@ -35,7 +35,9 @@
 #include "session.h"
 #include "balloon.h"
 #include "mystyle.h"
+#include "mylook.h"
 #include "wmprops.h"
+#include "desktop_category.h"
 #include "../libAfterImage/xpm.h"
 #include "../libAfterImage/char2uni.h"
 
@@ -90,106 +92,134 @@ void (*CloseOnExec)() = NULL ;
 struct ASSession *Session = NULL;          /* filenames of look, feel and background */
 struct ASEnvironment *Environment = NULL;
 
+struct ASDatabase    *Database = NULL;
+
+struct ASCategoryTree *StandardCategories = NULL ;
+struct ASCategoryTree *AfterStepCategories = NULL ;
+struct ASCategoryTree *KDECategories = NULL ;
+struct ASCategoryTree *GNOMECategories = NULL ;
+struct ASCategoryTree *SystemCategories = NULL ;
+struct ASCategoryTree *CombinedCategories = NULL ;
+
+
+
 /* names of AS functions - used all over the place  :*/
 
-#define FUNC_TERM(txt,len,func)         {TF_NO_MYNAME_PREPENDING,txt,len,TT_FUNCTION,func,NULL}
-#define FUNC_TERM2(flags,txt,len,func)  {TF_NO_MYNAME_PREPENDING|(flags),txt,len,TT_FUNCTION,func,NULL}
+#define FUNC_TERM(keyword,func)         {TF_NO_MYNAME_PREPENDING|TF_NAMED,keyword,sizeof(keyword)-1,TT_FUNCTION,func,NULL}
+#define FUNC_TERM2(flags,keyword,func)  {TF_NO_MYNAME_PREPENDING|TF_NAMED|(flags),keyword,sizeof(keyword)-1,TT_FUNCTION,func,NULL}
 
 TermDef       FuncTerms[F_FUNCTIONS_NUM + 1] = {
-	FUNC_TERM2 (NEED_NAME, "Nop", 3, F_NOP),   /* Nop      "name"|"" */
-	FUNC_TERM2 (NEED_NAME, "Title", 5, F_TITLE),	/* Title    "name"    */
-	FUNC_TERM ("Beep", 4, F_BEEP),			   /* Beep               */
-	FUNC_TERM ("Quit", 4, F_QUIT),			   /* Quit     ["name"] */
-	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Restart", 7, F_RESTART),	/* Restart "name" WindowManagerName */
-	FUNC_TERM ("Refresh", 7, F_REFRESH),	   /* Refresh  ["name"] */
+	FUNC_TERM2 (NEED_NAME, "Nop", F_NOP),   /* Nop      "name"|"" */
+	FUNC_TERM2 (NEED_NAME, "Title", F_TITLE),	/* Title    "name"    */
+	FUNC_TERM ("Beep", F_BEEP),			   /* Beep               */
+	FUNC_TERM ("Quit", F_QUIT),			   /* Quit     ["name"] */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Restart", F_RESTART),	/* Restart "name" WindowManagerName */
+	FUNC_TERM ("Refresh", F_REFRESH),	   /* Refresh  ["name"] */
 #ifndef NO_VIRTUAL
-	FUNC_TERM2 (USES_NUMVALS, "Scroll", 6, F_SCROLL),	/* Scroll     horiz vert */
-	FUNC_TERM2 (USES_NUMVALS, "GotoPage", 8, F_GOTO_PAGE),	/* GotoPage   x     y    */
-	FUNC_TERM ("TogglePage", 10, F_TOGGLE_PAGE),	/* TogglePage ["name"]   */
+	FUNC_TERM2 (USES_NUMVALS, "Scroll", F_SCROLL),	/* Scroll     horiz vert */
+	FUNC_TERM2 (USES_NUMVALS, "GotoPage", F_GOTO_PAGE),	/* GotoPage   x     y    */
+	FUNC_TERM ("TogglePage", F_TOGGLE_PAGE),	/* TogglePage ["name"]   */
 #endif
-	FUNC_TERM2 (USES_NUMVALS, "CursorMove", 10, F_MOVECURSOR),	/* CursorMove horiz vert */
-	FUNC_TERM2 (NEED_WINIFNAME, "WarpFore", 8, F_WARP_F),	/* WarpFore ["name" window_name] */
-	FUNC_TERM2 (NEED_WINIFNAME, "WarpBack", 8, F_WARP_B),	/* WarpBack ["name" window_name] */
-	FUNC_TERM2 (NEED_NAME | NEED_WINDOW, "Wait", 4, F_WAIT),	/* Wait      "name" window_name  */
-	FUNC_TERM2 (USES_NUMVALS, "Desk", 4, F_DESK),	/* Desk arg1 [arg2] */
-	FUNC_TERM2 (USES_NUMVALS, "GotoDeskViewport", 16, F_GOTO_DESKVIEWPORT),	/* GotoDeskViewport DESK+VX+VY */
+	FUNC_TERM2 (USES_NUMVALS, "CursorMove", F_MOVECURSOR),	/* CursorMove horiz vert */
+	FUNC_TERM2 (NEED_WINIFNAME, "WarpFore", F_WARP_F),	/* WarpFore ["name" window_name] */
+	FUNC_TERM2 (NEED_WINIFNAME, "WarpBack", F_WARP_B),	/* WarpBack ["name" window_name] */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Wait", F_WAIT),	/* Wait      "name" attributes  */
+	FUNC_TERM2 (USES_NUMVALS, "Desk", F_DESK),	/* Desk arg1 [arg2] */
+	FUNC_TERM2 (USES_NUMVALS, "GotoDeskViewport", F_GOTO_DESKVIEWPORT),	/* GotoDeskViewport DESK+VX+VY */
 #ifndef NO_WINDOWLIST
-	FUNC_TERM2 (USES_NUMVALS, "WindowList", 10, F_WINDOWLIST),	/* WindowList [arg1 arg2] */
+	FUNC_TERM2 (USES_NUMVALS, "WindowList", F_WINDOWLIST),	/* WindowList [arg1 arg2] */
 #endif
-	FUNC_TERM2 (NEED_NAME, "PopUp", 5, F_POPUP),	/* PopUp    "popup_name" [popup_name] */
-	FUNC_TERM2 (NEED_NAME, "Function", 8, F_FUNCTION),	/* Function "function_name" [function_name] */
-	FUNC_TERM ("MiniPixmap", 10, F_MINIPIXMAP),	/* MiniPixmap "name" */
-	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Exec", 4, F_EXEC),	/* Exec   "name" command */
-	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Module", 6, F_MODULE),	/* Module "name" command */
-	FUNC_TERM2 (NEED_NAME | NEED_CMD, "KillModuleByName", 16, F_KILLMODULEBYNAME),	/* KillModuleByName "name" module */
-	FUNC_TERM2 (NEED_NAME | NEED_CMD, "QuickRestart", 12, F_QUICKRESTART),	/* QuickRestart "name" what */
-	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Background", 10, F_CHANGE_BACKGROUND),	/* Background "name" file_name */
-	FUNC_TERM2 (NEED_NAME | NEED_CMD, "ChangeLook", 10, F_CHANGE_LOOK),	/* ChangeLook "name" file_name */
-	FUNC_TERM2 (NEED_NAME | NEED_CMD, "ChangeFeel", 10, F_CHANGE_FEEL),	/* ChangeFeel "name" file_name */
-    FUNC_TERM2 (NEED_NAME | NEED_CMD, "ChangeTheme", 11, F_CHANGE_THEME), /* ChangeTheme "name" file_name */
-    FUNC_TERM2 (NEED_NAME | NEED_CMD, "ChangeThemeFile", 15, F_CHANGE_THEME_FILE), /* "name" file_name */
-    FUNC_TERM2 (NEED_NAME | NEED_CMD, "ChangeColorscheme", 17, F_CHANGE_COLORSCHEME), /* ChangeColorscheme "name" file_name */
-    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallLook", 11, F_INSTALL_LOOK), /*  "name" file_name */
-    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallFeel", 11, F_INSTALL_FEEL), /*  "name" file_name */
-    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallBackground", 17, F_INSTALL_BACKGROUND), /* "name" file_name */
-    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallFont", 11, F_INSTALL_FONT), /* "name" file_name */
-    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallIcon", 11, F_INSTALL_ICON), /*  "name" file_name */
-    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallTile", 11, F_INSTALL_TILE), /*  "name" file_name */
-    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallThemeFile", 12, F_INSTALL_THEME_FILE), /*  "name" file_name */
-    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallColorscheme", 14, F_INSTALL_COLORSCHEME), /*  "name" file_name */
-	FUNC_TERM2 (NEED_NAME | NEED_CMD, "SaveWorkspace", 13, F_SAVE_WORKSPACE), /* SaveWorkspace "name" file_name */
-	FUNC_TERM2 (TF_SYNTAX_TERMINATOR, "EndFunction", 11, F_ENDFUNC),
-	FUNC_TERM2 (TF_SYNTAX_TERMINATOR, "EndPopup", 8, F_ENDPOPUP),
-	FUNC_TERM ("TakeScreenShot", 14, F_TAKE_SCREENSHOT),
-	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Test", 4, F_Test),
+	FUNC_TERM ("StopModuleList", F_STOPMODULELIST),	/* StopModuleList "name" */
+	FUNC_TERM ("RestartModuleList", F_RESTARTMODULELIST),	/* RestartModuleList "name" */
+	FUNC_TERM2 (NEED_NAME, "PopUp", F_POPUP),	/* PopUp    "popup_name" [popup_name] */
+	FUNC_TERM2 (NEED_NAME, "Function", F_FUNCTION),	/* Function "function_name" [function_name] */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Category", F_CATEGORY),	/* Category "function_name" category_name */
+	FUNC_TERM ("MiniPixmap", F_MINIPIXMAP),	/* MiniPixmap "name" */
+	FUNC_TERM ("SmallMiniPixmap", F_SMALL_MINIPIXMAP),	/* SmallMiniPixmap "name" */
+	FUNC_TERM ("LargeMiniPixmap", F_SMALL_MINIPIXMAP),	/* LargeMiniPixmap "name" */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Exec", F_EXEC),	/* Exec   "name" command */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Module", F_MODULE),	/* Module "name" command */
+    FUNC_TERM2 (NEED_NAME | NEED_CMD, "ExecInTerm", F_ExecInTerm),   /* ExecInTerm   "name" command */
+    FUNC_TERM2 (NEED_NAME | NEED_CMD, "ExecBrowser", F_ExecBrowser),   /* ExecBrowser   "name" url */
+    FUNC_TERM2 (NEED_NAME | NEED_CMD, "ExecEditor",  F_ExecEditor),    /* ExecEditor   "name" filename */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "KillModuleByName", F_KILLMODULEBYNAME),	/* KillModuleByName "name" module */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "RestartModuleByName", F_RESTARTMODULEBYNAME),	/* RestartModuleByName "name" module */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "KillAllModulesByName", F_KILLALLMODULESBYNAME),	/* KillAllModulesByName "name" module */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "QuickRestart", F_QUICKRESTART),	/* QuickRestart "name" what */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Background", F_CHANGE_BACKGROUND),	/* Background "name" file_name */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "BackgroundForeign", F_CHANGE_BACKGROUND_FOREIGN),	/* BackgroundForeign "name" file_name */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "ChangeLook", F_CHANGE_LOOK),	/* ChangeLook "name" file_name */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "ChangeFeel", F_CHANGE_FEEL),	/* ChangeFeel "name" file_name */
+    FUNC_TERM2 (NEED_NAME | NEED_CMD, "ChangeTheme", F_CHANGE_THEME), /* ChangeTheme "name" file_name */
+    FUNC_TERM2 (NEED_NAME | NEED_CMD, "ChangeThemeFile", F_CHANGE_THEME_FILE), /* "name" file_name */
+    FUNC_TERM2 (NEED_NAME | NEED_CMD, "ChangeColorscheme", F_CHANGE_COLORSCHEME), /* ChangeColorscheme "name" file_name */
+    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallLook", F_INSTALL_LOOK), /*  "name" file_name */
+    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallFeel", F_INSTALL_FEEL), /*  "name" file_name */
+    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallBackground", F_INSTALL_BACKGROUND), /* "name" file_name */
+    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallFont", F_INSTALL_FONT), /* "name" file_name */
+    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallIcon", F_INSTALL_ICON), /*  "name" file_name */
+    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallTile", F_INSTALL_TILE), /*  "name" file_name */
+    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallThemeFile", F_INSTALL_THEME_FILE), /*  "name" file_name */
+    FUNC_TERM2 (NEED_NAME | NEED_CMD, "InstallColorscheme", F_INSTALL_COLORSCHEME), /*  "name" file_name */
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "SaveWorkspace", F_SAVE_WORKSPACE), /* SaveWorkspace "name" file_name */
+	FUNC_TERM  ("SignalReloadGTKRCFile", F_SIGNAL_RELOAD_GTK_RCFILE),
+	FUNC_TERM  ("KIPCsendMessageAll", F_KIPC_SEND_MESSAGE_ALL),
+	FUNC_TERM2 (TF_SYNTAX_TERMINATOR, "EndFunction", F_ENDFUNC),
+	FUNC_TERM2 (TF_SYNTAX_TERMINATOR, "EndPopup", F_ENDPOPUP),
+	FUNC_TERM ("TakeScreenShot", F_TAKE_SCREENSHOT),
 
+	FUNC_TERM2 (NEED_CMD, "Set", F_SET),	/* Set "name" <variable>=<value> */
+
+	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Test", F_Test),
+	
 	/* this functions require window as aparameter */
-	FUNC_TERM ("&nonsense&", 10, F_WINDOW_FUNC_START),	/* not really a command */
-	FUNC_TERM2 (USES_NUMVALS, "Move", 4, F_MOVE),	/* Move     ["name"] [whereX whereY] */
-	FUNC_TERM2 (USES_NUMVALS, "Resize", 6, F_RESIZE),	/* Resize   ["name"] [toWidth toHeight] */
-	FUNC_TERM ("Raise", 5, F_RAISE),		   /* Raise    ["name"] */
-	FUNC_TERM ("Lower", 5, F_LOWER),		   /* Lower    ["name"] */
-	FUNC_TERM ("RaiseLower", 10, F_RAISELOWER),	/* RaiseLower ["name"] */
-	FUNC_TERM ("PutOnTop", 8, F_PUTONTOP),	   /* PutOnTop  */
-	FUNC_TERM ("PutOnBack", 9, F_PUTONBACK),   /* PutOnBack */
-	FUNC_TERM2 (USES_NUMVALS, "SetLayer", 8, F_SETLAYER),	/* SetLayer    layer */
-	FUNC_TERM2 (USES_NUMVALS, "ToggleLayer", 11, F_TOGGLELAYER),	/* ToggleLayer layer1 layer2 */
-	FUNC_TERM ("Shade", 5, F_SHADE),		   /* Shade    ["name"] */
-	FUNC_TERM ("Delete", 6, F_DELETE),		   /* Delete   ["name"] */
-	FUNC_TERM ("Destroy", 7, F_DESTROY),	   /* Destroy  ["name"] */
-	FUNC_TERM ("Close", 5, F_CLOSE),		   /* Close    ["name"] */
-	FUNC_TERM ("Iconify", 7, F_ICONIFY),	   /* Iconify  ["name"] value */
-	FUNC_TERM2 (USES_NUMVALS, "Maximize", 8, F_MAXIMIZE),	/* Maximize ["name"] [hori vert] */
-	FUNC_TERM ("Stick", 5, F_STICK),		   /* Stick    ["name"] */
-	FUNC_TERM ("Focus", 5, F_FOCUS),		   /* Focus */
-	FUNC_TERM2 (NEED_WINIFNAME, "ChangeWindowUp", 14, F_CHANGEWINDOW_UP),	/* ChangeWindowUp   ["name" window_name ] */
-	FUNC_TERM2 (NEED_WINIFNAME, "ChangeWindowDown", 16, F_CHANGEWINDOW_DOWN),	/* ChangeWindowDown ["name" window_name ] */
-    FUNC_TERM2 (NEED_WINIFNAME, "GoToBookmark", 12, F_GOTO_BOOKMARK),   /* GoToBookmark ["name" window_bookmark ] */
-	FUNC_TERM ("GetHelp", 7, F_GETHELP),	   /* */
-	FUNC_TERM ("PasteSelection", 14, F_PASTE_SELECTION),	/* */
-	FUNC_TERM2 (USES_NUMVALS, "WindowsDesk", 11, F_CHANGE_WINDOWS_DESK),	/* WindowDesk "name" new_desk */
-    FUNC_TERM ("BookmarkWindow", 14, F_BOOKMARK_WINDOW),    /* BookmarkWindow "name" new_bookmark */
-    FUNC_TERM ("PinMenu", 7, F_PIN_MENU),    /* PinMenu ["name"] */
-	FUNC_TERM ("TakeWindowShot", 14, F_TAKE_WINDOWSHOT),
-	FUNC_TERM ("TakeFrameShot", 13, F_TAKE_FRAMESHOT),
+	FUNC_TERM ("&nonsense&", F_WINDOW_FUNC_START),	/* not really a command */
+	FUNC_TERM2 (USES_NUMVALS, "Move", F_MOVE),	/* Move     ["name"] [whereX whereY] */
+	FUNC_TERM2 (USES_NUMVALS, "Resize", F_RESIZE),	/* Resize   ["name"] [toWidth toHeight] */
+	FUNC_TERM ("Raise", F_RAISE),		   /* Raise    ["name"] */
+	FUNC_TERM ("Lower", F_LOWER),		   /* Lower    ["name"] */
+	FUNC_TERM ("RaiseLower", F_RAISELOWER),	/* RaiseLower ["name"] */
+	FUNC_TERM ("PutOnTop", F_PUTONTOP),	   /* PutOnTop  */
+	FUNC_TERM ("PutOnBack", F_PUTONBACK),   /* PutOnBack */
+	FUNC_TERM2 (USES_NUMVALS, "SetLayer", F_SETLAYER),	/* SetLayer    layer */
+	FUNC_TERM2 (USES_NUMVALS, "ToggleLayer", F_TOGGLELAYER),	/* ToggleLayer layer1 layer2 */
+	FUNC_TERM ("Shade", F_SHADE),		   /* Shade    ["name"] */
+	FUNC_TERM ("Delete", F_DELETE),		   /* Delete   ["name"] */
+	FUNC_TERM ("Destroy", F_DESTROY),	   /* Destroy  ["name"] */
+	FUNC_TERM ("Close", F_CLOSE),		   /* Close    ["name"] */
+	FUNC_TERM ("Iconify", F_ICONIFY),	   /* Iconify  ["name"] value */
+	FUNC_TERM2 (USES_NUMVALS, "Maximize", F_MAXIMIZE),	/* Maximize ["name"] [hori vert] */
+	FUNC_TERM ("Fullscreen", F_FULLSCREEN),	/* Maximize ["name"] [hori vert] */
+	FUNC_TERM ("Stick", F_STICK),		   /* Stick    ["name"] */
+	FUNC_TERM ("Focus", F_FOCUS),		   /* Focus */
+	FUNC_TERM2 (NEED_WINIFNAME, "ChangeWindowUp", F_CHANGEWINDOW_UP),	/* ChangeWindowUp   ["name" window_name ] */
+	FUNC_TERM2 (NEED_WINIFNAME, "ChangeWindowDown", F_CHANGEWINDOW_DOWN),	/* ChangeWindowDown ["name" window_name ] */
+    FUNC_TERM2 (NEED_WINIFNAME, "GoToBookmark", F_GOTO_BOOKMARK),   /* GoToBookmark ["name" window_bookmark ] */
+	FUNC_TERM ("GetHelp", F_GETHELP),	   /* */
+	FUNC_TERM ("PasteSelection", F_PASTE_SELECTION),	/* */
+	FUNC_TERM2 (USES_NUMVALS, "WindowsDesk", F_CHANGE_WINDOWS_DESK),	/* WindowDesk "name" new_desk */
+    FUNC_TERM ("BookmarkWindow", F_BOOKMARK_WINDOW),    /* BookmarkWindow "name" new_bookmark */
+    FUNC_TERM ("PinMenu", F_PIN_MENU),    /* PinMenu ["name"] */
+	FUNC_TERM ("TakeWindowShot", F_TAKE_WINDOWSHOT),
+	FUNC_TERM ("TakeFrameShot", F_TAKE_FRAMESHOT),
+	FUNC_TERM ("SwallowWindow", F_SWALLOW_WINDOW),  /* SwallowWindow "name" module_name */ 
 	/* end of window functions */
 	/* these are commands  to be used only by modules */
-	FUNC_TERM ("&nonsense&", 10, F_MODULE_FUNC_START),	/* not really a command */
-	FUNC_TERM ("Send_WindowList", 15, F_SEND_WINDOW_LIST),	/* */
-	FUNC_TERM ("SET_MASK", 8, F_SET_MASK),	   /* SET_MASK  mask */
-    FUNC_TERM2 (NEED_NAME, "SET_NAME", 8, F_SET_NAME),   /* SET_NAME  name */
-	FUNC_TERM ("UNLOCK", 6, F_UNLOCK),		   /* UNLOCK    1  */
-	FUNC_TERM ("SET_FLAGS", 9, F_SET_FLAGS),   /* SET_FLAGS flags */
+	FUNC_TERM ("&nonsense&", F_MODULE_FUNC_START),	/* not really a command */
+	FUNC_TERM ("Send_WindowList", F_SEND_WINDOW_LIST),	/* */
+	FUNC_TERM ("SET_MASK", F_SET_MASK),	   /* SET_MASK  mask lock_mask */
+    FUNC_TERM2 (NEED_NAME, "SET_NAME", F_SET_NAME),   /* SET_NAME  name */
+	FUNC_TERM ("UNLOCK", F_UNLOCK),		   /* UNLOCK    1  */
+	FUNC_TERM ("SET_FLAGS", F_SET_FLAGS),   /* SET_FLAGS flags */
 	/* these are internal commands */
-	FUNC_TERM ("&nonsense&", 10, F_INTERNAL_FUNC_START),	/* not really a command */
-	FUNC_TERM ("&raise_it&", 10, F_RAISE_IT),  /* should not be used by user */
+	FUNC_TERM ("&nonsense&", F_INTERNAL_FUNC_START),	/* not really a command */
+	FUNC_TERM ("&raise_it&", F_RAISE_IT),  /* should not be used by user */
     /* wharf functions : */
     {TF_NO_MYNAME_PREPENDING, "Folder", 6, TT_FUNCTION, F_Folder, NULL},
-    {TF_NO_MYNAME_PREPENDING | NEED_NAME | NEED_CMD, "Swallow", 7, TT_FUNCTION, F_Swallow, NULL},
-    {TF_NO_MYNAME_PREPENDING | NEED_NAME | NEED_CMD, "MaxSwallow", 10, TT_FUNCTION, F_MaxSwallow, NULL},
-    {TF_NO_MYNAME_PREPENDING | NEED_NAME | NEED_CMD, "SwallowModule", 13, TT_FUNCTION, F_SwallowModule, NULL},
-    {TF_NO_MYNAME_PREPENDING | NEED_NAME | NEED_CMD, "MaxSwallowModule", 16, TT_FUNCTION, F_MaxSwallowModule, NULL},
-    FUNC_TERM2 (NEED_NAME | NEED_CMD, "DropExec", 8, F_DropExec),   /* DropExec   "name" command */
+    {TF_NO_MYNAME_PREPENDING | NEED_NAME | NEED_CMD| TF_NAMED, "Swallow", 7, TT_FUNCTION, F_Swallow, NULL},
+    {TF_NO_MYNAME_PREPENDING | NEED_NAME | NEED_CMD| TF_NAMED, "MaxSwallow", 10, TT_FUNCTION, F_MaxSwallow, NULL},
+    {TF_NO_MYNAME_PREPENDING | NEED_NAME | NEED_CMD| TF_NAMED, "SwallowModule", 13, TT_FUNCTION, F_SwallowModule, NULL},
+    {TF_NO_MYNAME_PREPENDING | NEED_NAME | NEED_CMD| TF_NAMED, "MaxSwallowModule", 16, TT_FUNCTION, F_MaxSwallowModule, NULL},
     {TF_NO_MYNAME_PREPENDING, "Size", 4, TT_FUNCTION, F_Size, NULL},
     {TF_NO_MYNAME_PREPENDING, "Transient", 9, TT_FUNCTION, F_Transient, NULL},
 
@@ -197,21 +227,45 @@ TermDef       FuncTerms[F_FUNCTIONS_NUM + 1] = {
 };
 
 struct SyntaxDef FuncSyntax = {
-    '\0',
-    '\n',
-	FuncTerms,
-	0,										   /* use default hash size */
-    ' ',
-	"",
-	"\t",
+    '\0',   '\n',	FuncTerms,
+	0, 		' ',	"",			"\t",
 	"AfterStep Function",
 	"Functions",
 	"built in AfterStep functions",
-	NULL,
-	0
+	NULL,	0
+};
+SyntaxDef     PopupFuncSyntax = {
+	'\n',	'\0',	FuncTerms,
+	0, 		' ',	"\t",		"\t",
+    "Popup/Complex function definition",
+	"Popup",
+	"",	
+	NULL,	0
 };
 
+
+
 struct SyntaxDef     *pFuncSyntax = &FuncSyntax;
+struct SyntaxDef     *pPopupFuncSyntax = &PopupFuncSyntax;
+
+TermDef      *
+func2fterm (FunctionCode func, int quiet)
+{
+	register int  i;
+
+	/* in most cases that should work : */
+	if( func < F_FUNCTIONS_NUM )
+		if (FuncTerms[func].id == func)
+			return &(FuncTerms[func]);
+
+	/* trying fallback if it did not : */
+	for (i = 0; i < F_FUNCTIONS_NUM; i++)
+		if (FuncTerms[i].id == func)
+			return &(FuncTerms[i]);
+
+	/* something terribly wrong has happened : */
+	return NULL;
+}
 
 /************************************************************************************/
 /* Command Line Processing/ App initialization here :                               */
@@ -238,7 +292,7 @@ SetMyName (char *argv0)
 
 /* If you change/add options please change InitMyApp below and option flags in aftersteplib.h */
 
-CommandLineOpts as_standard_cmdl_options[20] =
+CommandLineOpts as_standard_cmdl_options[STANDARD_CMDL_OPTS_NUM] =
 {
 #define  SHOW_VERSION   0
 #define  SHOW_CONFIG    1
@@ -248,7 +302,7 @@ CommandLineOpts as_standard_cmdl_options[20] =
 /* 2*/{"h", "help",    "Display uasge information and stop",   NULL, handler_show_info, NULL, SHOW_USAGE},
 /* 3*/{NULL,"debug",   "Debugging: Run in Synchronous mode",  NULL, handler_set_flag, &(as_app_args.flags), ASS_Debugging},
 /* 4*/{"s", "single",  "Run on single screen only", NULL, handler_set_flag, &(as_app_args.flags), ASS_SingleScreen},
-/* 5*/{"r", "restart", "Run as if it was restarted","same as regular startup, only runs RestartFunction\ninstead of InitFunction",
+/* 5*/{"r", "restart", "Run as if it was restarted","same as regular startup, only \nruns RestartFunctioninstead of InitFunction",
                                                            handler_set_flag, &(as_app_args.flags), ASS_Restarting},
 #define OPTION_HAS_ARGS     6
 /* 6*/{"d", "display", "Specify what X display we should connect to","Overrides $DISPLAY environment variable",
@@ -275,10 +329,16 @@ CommandLineOpts as_standard_cmdl_options[20] =
 /*16*/{NULL, "trace-func","Debugging: Trace calls to a function with requested name", NULL,
                                                            handler_set_string, &(as_app_args.trace_calls), 0, CMO_HasArgs },
 #endif
-/*17*/{"l", "log","Save all output into the file instead of printing it to console", NULL,
+/*17*/{"l", "log","Save all output into the file", "(instead of printing it to console)",
                                                            handler_set_string, &(as_app_args.log_file), 0, CMO_HasArgs },
-/*18*/{"L", "locale","Set language locale to be used while displaying text", NULL,
-                                                           handler_set_string, &(as_app_args.locale), 0, CMO_HasArgs },
+/*18*/{"L", "locale","Set language locale", "to be used while displaying text",
+                                                           handler_set_dup_string, &(as_app_args.locale), 0, CMO_HasArgs },
+/*19*/{NULL, "myname","Overrides module name", "will be used while parsing config files\nand reporting to AfterStep",
+                                                           handler_set_string, &(MyName), 0, CMO_HasArgs },
+/*20*/{NULL, "geometry","Overrides module's geometry", NULL,
+                                                           handler_set_geometry, &(as_app_args.geometry), 0, CMO_HasArgs },
+/*21*/{NULL, "gravity","Overrides module's gravity", NULL,
+                                                           handler_set_gravity, &(as_app_args.gravity), 0, CMO_HasArgs },
       {NULL, NULL, NULL, NULL, NULL, NULL, 0 }
 };
 
@@ -383,6 +443,18 @@ void  handler_set_string( char *argv, void *trg, long param )
     if( argv )
         *s = argv;
 }
+
+void  handler_set_dup_string( char *argv, void *trg, long param )
+{
+    register char **s = trg;
+    if( argv )
+	{
+    	if( *s ) 
+			free( *s );
+	    *s = mystrdup(argv);
+	}
+}
+
 void  handler_set_int( char *argv, void *trg, long param )
 {
     register int *i = trg ;
@@ -390,6 +462,34 @@ void  handler_set_int( char *argv, void *trg, long param )
         *i = atoi(argv);
 }
 
+void  handler_set_geometry( char *argv, void *trg, long param )
+{
+    register ASGeometry *geom = trg ;
+    if( argv )
+	{
+		memset( geom, 0x00, sizeof(ASGeometry));
+		parse_geometry (argv, &(geom->x), &(geom->y), &(geom->width), &(geom->height), &(geom->flags) );
+	}
+}
+
+void  handler_set_gravity( char *argv, void *trg, long param )
+{
+    register int *i = trg ;
+    *i = ForgetGravity ;
+	if( argv )
+	{
+
+		if( mystrncasecmp( argv, "NorthWest", 9 )== 0 )	  *i = NorthWestGravity ; 
+		else if( mystrncasecmp( argv, "SouthWest", 9 )== 0 )	  *i = SouthWestGravity ; 
+		else if( mystrncasecmp( argv, "NorthEasth", 10 )== 0 )	*i = NorthEastGravity ; 
+		else if( mystrncasecmp( argv, "SouthEast", 9 )== 0 )	  *i = SouthEastGravity ; 
+		else if( mystrncasecmp( argv, "Center", 6 )== 0 )	  *i = CenterGravity ; 
+		else if( mystrncasecmp( argv, "Static", 6 )== 0 )	  *i = StaticGravity ; 
+		else show_warning( "unknown gravity type \"%s\". Use one of: NorthWest, SouthWest, NorthEast, SouthEast, Center, Static", argv );
+	}
+}
+
+
 int
 match_command_line_opt( char *argvi, CommandLineOpts *options )
 {
@@ -438,6 +538,13 @@ void ASDeadPipe( int nonsense )
 		exit (0);
 }	 
 
+static char **AS_environ = NULL ; 
+
+void override_environ( char **envp )
+{
+	AS_environ = envp ; 	
+}	 
+
 void
 InitMyApp (  const char *app_class, int argc, char **argv, void (*version_func) (void), void (*custom_usage_func) (void), ASFlagType opt_mask )
 {
@@ -455,7 +562,7 @@ InitMyApp (  const char *app_class, int argc, char **argv, void (*version_func)
 
     memset( &as_app_args, 0x00, sizeof(ASProgArgs) );
 	as_app_args.locale = mystrdup(AFTER_LOCALE);
-	if (strlen(as_app_args.locale) == 0)
+	if (as_app_args.locale[0] == '\0')
 	{
 		free( as_app_args.locale );
 		as_app_args.locale = mystrdup(getenv("LANG"));
@@ -470,6 +577,7 @@ InitMyApp (  const char *app_class, int argc, char **argv, void (*version_func)
 
 
     ASDefaultScr = safecalloc(1, sizeof(ScreenInfo));
+	init_ScreenInfo(ASDefaultScr);
 
     if( argc > 0 && argv )
     {
@@ -478,7 +586,7 @@ InitMyApp (  const char *app_class, int argc, char **argv, void (*version_func)
 		as_app_args.saved_argc = argc ;
 		as_app_args.saved_argv = safecalloc( argc, sizeof(char*));
 		for( i = 0 ; i < argc ; ++i )
-			as_app_args.saved_argv[i] = argv[i] ;
+			as_app_args.saved_argv[i] = mystrdup(argv[i]) ;
 
         SetMyName( argv[0] );
 
@@ -502,15 +610,17 @@ InitMyApp (  const char *app_class, int argc, char **argv, void (*version_func)
         }
     }
 
-    fd_width = get_fd_width ();
-
-    if (FuncSyntax.term_hash == NULL)
-		PrepareSyntax (&FuncSyntax);
     set_output_threshold( as_app_args.verbosity_level );
     if(as_app_args.log_file)
 		if( freopen( as_app_args.log_file, get_flags( as_app_args.flags, ASS_Restarting)?"a":"w", stderr ) == NULL )
 	    	show_system_error( "failed to redirect output into file \"%s\"", as_app_args.log_file );
 
+
+    fd_width = get_fd_width ();
+
+    if (FuncSyntax.term_hash == NULL)
+		PrepareSyntax (&FuncSyntax);
+
 	if( as_app_args.locale && strlen(as_app_args.locale) > 0)
 	{
 		as_set_charset( parse_charset_name( as_app_args.locale ));
@@ -532,8 +642,16 @@ InitMyApp (  const char *app_class, int argc, char **argv, void (*version_func)
     trace_enable_function(as_app_args.trace_calls);
 #endif
 
-	asxml_var_insert("minipixmap.width", 48);
-	asxml_var_insert("minipixmap.height", 48);
+	asxml_var_insert(ASXMLVAR_IconButtonWidth, 64);
+	asxml_var_insert(ASXMLVAR_IconButtonHeight, 64);
+	asxml_var_insert(ASXMLVAR_IconWidth, 48);
+	asxml_var_insert(ASXMLVAR_IconHeight, 48);
+	asxml_var_insert(ASXMLVAR_MinipixmapWidth, 24);
+	asxml_var_insert(ASXMLVAR_MinipixmapHeight, 24);
+	asxml_var_insert(ASXMLVAR_TitleFontSize, 14);
+	asxml_var_insert(ASXMLVAR_MenuFontSize, 16);
+	asxml_var_insert(ASXMLVAR_MenuShowMinipixmaps, 1);
+	asxml_var_insert(ASXMLVAR_MenuShowUnavailable, 1);
 
 }
 
@@ -547,10 +665,99 @@ free_func_hash ()
 }
 
 /*********** end command line parsing **************************/
+static char *_as_known_terms[]	= 
+{	
+	"aterm",
+	"rxvt  -tr -fg yellow -bg black",
+	"eterm -tr -tint blue -fg yellow -bg black",
+	"xterm -fg yellow -bg blue",
+	NULL 
+};
+static char *_as_known_browsers[]	= 
+{
+	"$BROWSER",
+	"firefox",
+	"x-www-browser",           /* don't like default debian selection of konqueror */
+	"mozilla-firefox",
+	"mozilla",
+	"opera",
+	NULL				 
+};
+static char *_as_known_editors[]	= 
+{
+	"$EDITOR",
+	"nedit",
+	"xemacs",
+	"gedit",
+	"kedit",		 
+	"kate",
+	NULL
+};
+
+static char **_as_known_tools[ASTool_Count] = 
+{
+	_as_known_terms,
+	_as_known_browsers,
+	_as_known_editors
+};
+
+static char *_as_tools_name[ASTool_Count] = 
+{
+	"Terminal",
+	"Browser",
+	"Editor"	  
+};	 
+
+	 
+char *as_get_default_tool(ASToolType type)
+{
+	int i ;							
+	for( i = 0 ; _as_known_tools[type][i] ; ++i ) 
+	{
+		char *tmp = _as_known_tools[type][i] ;
+		if( tmp[0] == '$' ) 
+			tmp = copy_replace_envvar( tmp );
+	 	else
+			tmp = mystrdup(tmp);				   
+		if( is_executable_in_path (tmp) )
+			return tmp;
+		free( tmp );
+	}
+	return NULL;
+}
+
+void
+set_environment_tool_from_list( ASEnvironment *e, ASToolType type, char ** list, int list_len )
+{
+	int i ;
+	destroy_string( &(e->tool_command[type]) );
+	for( i = 0 ; i < list_len ; ++i ) 
+		if( list[i] )
+		{
+			char *tmp = list[i] ;
+			if( tmp[0] == '$' ) 
+				tmp = copy_replace_envvar( tmp );
+	 		else
+				tmp = mystrdup(tmp);				   
+			if( is_executable_in_path( tmp ) ) 
+			{	
+				e->tool_command[type] = tmp;
+				break;
+			}else
+			{
+				show_warning( "%s command %s is not in the path", _as_tools_name[type], tmp );
+				free(tmp);	  
+			}
+		}	 
+	if( e->tool_command[type] == NULL ) 
+		e->tool_command[type] = as_get_default_tool(type);
+	show_progress( "%s is set to: \"%s\"", _as_tools_name[type], e->tool_command[type]?e->tool_command[type]:"none" );
+}
 
 ASEnvironment *
 make_default_environment()
 {
+	int i;
 	ASEnvironment *e = safecalloc( 1, sizeof(ASEnvironment) );
 	static const char *default_pixmap_path_format =
 		"%s/desktop/icons/:"
@@ -608,6 +815,14 @@ make_default_environment()
 	sprintf( e->cursor_path, default_cursor_path_format,
 	                         AFTER_DIR,
 							 AFTER_SHAREDIR );
+
+	for( i = 0 ; i < ASTool_Count ; ++i )
+		e->tool_command[i] = as_get_default_tool(i);	 
+
+	/* by default - don't do overwrite gtkrc files so to not aggrave people */
+	e->gtkrc_path = NULL ;  /* make_session_rc_file(Session, GTKRC_FILE); */
+  	e->gtkrc20_path = NULL ; /* make_session_rc_file(Session, GTKRC20_FILE) ; */
+
 	return e;
 }
 
@@ -619,6 +834,7 @@ destroy_asenvironment( ASEnvironment **penv )
 		ASEnvironment *e = *penv ;
 		if( e )
 		{
+			int i ;
 			if( e->module_path )
 				free( e->module_path );
 			if( e->audio_path )
@@ -631,6 +847,11 @@ destroy_asenvironment( ASEnvironment **penv )
 				free( e->font_path );
 			if( e->cursor_path )
 				free( e->cursor_path );
+			for ( i = 0 ; i < ASTool_Count ; ++i )
+				destroy_string( &(e->tool_command[i])) ;
+			
+			destroy_string( &(e->gtkrc_path)) ;
+			destroy_string( &(e->gtkrc20_path)) ;
 
 			free( e ) ;
 			*penv = NULL ;
@@ -638,53 +859,120 @@ destroy_asenvironment( ASEnvironment **penv )
 	}
 }
 
+/*
+ * Initialize database variables
+ */
+
+void
+destroy_asdatabase()
+{
+	if( Database ) 
+	    destroy_asdb( &Database );
+    /* XResources : */
+    destroy_user_database();
+}
+
+ASDesktopCategory *
+name2desktop_category( const char *name, ASCategoryTree **tree_return ) 
+{
+	ASCategoryTree *ct = CombinedCategories ; 
+	int offset = 0 ;
+/*	fprintf( stderr, __FUNCTION__ ": checking \"%s\" (AfterSTep categories = %p)\n", name, AfterStepCategories );*/
+	
+	if( !mystrncasecmp (name, "AfterStep:", 10) )
+	{	
+		ct = AfterStepCategories ; 
+		offset = 10 ; 
+	}else if( !mystrncasecmp (name, "KDE:", 4) )
+	{	
+		ct = KDECategories ; 
+		offset = 4 ; 
+	}else if( !mystrncasecmp (name, "GNOME:", 6) )
+	{	
+		ct = GNOMECategories ; 
+		offset = 6 ; 
+	}else if( !mystrncasecmp (name, "SYSTEM:", 7) )
+	{	
+		ct = SystemCategories ;
+		offset = 7 ; 
+	}else if( !mystrncasecmp (name, "COMBINED:", 9) )
+	{	
+		ct = CombinedCategories; 
+		offset = 9 ;
+	}
+	if( tree_return ) 
+		*tree_return = ct ;
+	return fetch_desktop_category( ct, name+offset );
+}
+
 void
 InitSession()
 {
     /* initializing our dirs names */
-    Session = GetNCASSession(ASDefaultScr, as_app_args.override_home, as_app_args.override_share);
-    if( as_app_args.override_config )
-        set_session_override( Session, as_app_args.override_config, 0 );
-    if( as_app_args.override_look )
-        set_session_override( Session, as_app_args.override_look, F_CHANGE_LOOK );
-    if( as_app_args.override_feel )
-        set_session_override( Session, as_app_args.override_feel, F_CHANGE_FEEL );
+	if( Session == NULL ) 
+	{	
+    	Session = GetNCASSession(ASDefaultScr, as_app_args.override_home, as_app_args.override_share);
+    	if( as_app_args.override_config )
+        	set_session_override( Session, as_app_args.override_config, 0 );
+    	if( as_app_args.override_look )
+        	set_session_override( Session, as_app_args.override_look, F_CHANGE_LOOK );
+    	if( as_app_args.override_feel )
+        	set_session_override( Session, as_app_args.override_feel, F_CHANGE_FEEL );
+	}
 }
 
+void
+free_as_app_args()
+{
+	int i;
+	for( i = 0 ; i < as_app_args.saved_argc ; ++i ) 
+		if( as_app_args.saved_argv[i] )
+			free( as_app_args.saved_argv[i] );
+    free( as_app_args.saved_argv );
+	as_app_args.saved_argv = NULL ;
+
+	destroy_string( &(as_app_args.locale) );
+
+}
 
 void
 FreeMyAppResources()
 {
-	
-    balloon_init (True);
+	cleanup_default_balloons();
+	destroy_asdatabase();
     mystyle_destroy_all();
+    mylook_init( &(ASDefaultScr->Look), True, ASFLAGS_EVERYTHING );
     destroy_image_manager( ASDefaultScr->image_manager, False );
     destroy_font_manager( ASDefaultScr->font_manager, False );
     clientprops_cleanup ();
     destroy_wmprops (ASDefaultScr->wmprops, False);
     wmprops_cleanup ();
     free_func_hash();
+	flush_keyword_ids();
     purge_asimage_registry();
-    flush_ashash_memory_pool();
+	asxml_var_cleanup();
+	custom_color_cleanup();
+    build_xpm_colormap( NULL );
     destroy_screen_gcs(ASDefaultScr);
 	if( ASDefaultScr->RootImage ) 
 	{	
 		safe_asimage_destroy( ASDefaultScr->RootImage );
 		ASDefaultScr->RootImage = NULL ;
 	}
-	asxml_var_cleanup();
 	destroy_asvisual( ASDefaultScr->asv, False );
-	custom_color_cleanup();
-    flush_asbidirlist_memory_pool();
-    free( as_app_args.saved_argv );
+	free_as_app_args();
     destroy_assession( Session );
+	Session = NULL ;
 	destroy_asenvironment( &Environment );
-    build_xpm_colormap( NULL );
 	is_executable_in_path ( NULL );
 #ifdef XSHMIMAGE
 	flush_shm_cache();
 #endif
 	free( ASDefaultScr );
+	flush_default_asstorage();
+    flush_asbidirlist_memory_pool();
+	flush_ashash_memory_pool();
+	
 }
 
 
@@ -711,6 +999,7 @@ as_sigchild_handler (int signum)
 	while ((pid = WAIT_CHILDREN (&status)) > 0 )
 	{
 		register int i ;
+		LOCAL_DEBUG_OUT( "pid = %d", pid );
 		for( i = 0; i < MAX_SINGLETONS_NUM ; i++ )
 			if( pid == as_singletons[i] )
 			{
@@ -800,13 +1089,76 @@ spawn_child( const char *cmd, int singleton_id, int screen, Window w, int contex
     {/* we get here only in child process. We now need to spawn new proggy here: */
         int len;
         char *display = mystrdup(XDisplayString (dpy));
+		char **envp ; 
         register char *ptr ;
 
         char *cmdl;
         char *arg, *screen_str = NULL, *w_str = NULL, *context_str = NULL ;
+		int env_s = 0;
+		char **envvars = AS_environ ; 
+		int font_path_slot = -1, image_path_slot = -1 ;
+
         va_list ap;
 		LOCAL_DEBUG_OUT( "dpy = %p, DisplayString = \"%s\"", dpy, display );
 		LOCAL_DEBUG_OUT( "pid(%d), entered child process to spawn ...", pid );
+
+#if HAVE_DECL_ENVIRON
+		if( envvars == NULL ) 
+		{
+			envvars = environ ;
+		}	 
+#else
+/* how the hell could we get environment otherwise ? */
+#endif
+		if( envvars ) 
+		{
+			int font_path_len = strlen(ASFONT_PATH_ENVVAR);
+			int image_path_len = strlen(ASIMAGE_PATH_ENVVAR);
+			for( env_s = 0  ; envvars[env_s] != NULL ; ++env_s )
+			{
+				if( font_path_slot < 0 && strlen(envvars[env_s]) > font_path_len ) 
+					if(strncmp(envvars[env_s], ASFONT_PATH_ENVVAR, font_path_len)==0)
+						font_path_slot = env_s ; 
+				if( image_path_slot < 0 && strlen(envvars[env_s]) > image_path_len) 
+					if(strncmp(envvars[env_s], ASIMAGE_PATH_ENVVAR, image_path_len)==0)
+						image_path_slot = env_s ;
+			}	
+		}
+		if( font_path_slot < 0 ) 
+			++env_s ;
+		if( image_path_slot  < 0 ) 
+			++env_s ;
+		envp = safecalloc( env_s+2, sizeof(char*));
+
+		/* environment variabless to pass to child process */
+		if( envvars ) 
+			for( env_s = 0  ; envvars[env_s] != NULL ; ++env_s )
+				envp[env_s] = envvars[env_s] ;	
+        
+		envp[env_s] = safemalloc(8+strlen(display)+1);
+		sprintf( envp[env_s], "DISPLAY=%s", display );
+		++env_s ;
+		if( Environment ) 
+		{
+			if( Environment->pixmap_path != NULL ) 
+			{	
+				int slot_no = image_path_slot ; 
+				if( slot_no < 0 ) 
+					slot_no = env_s++ ; 
+					
+				envp[slot_no] = safemalloc(strlen( ASIMAGE_PATH_ENVVAR ) + 1 + strlen(Environment->pixmap_path)+1) ;
+				sprintf( envp[slot_no], "%s=%s", ASIMAGE_PATH_ENVVAR, Environment->pixmap_path );
+			}
+			if( Environment->font_path ) 
+			{	
+				int slot_no = font_path_slot ; 
+				if( slot_no < 0 ) 
+					slot_no = env_s++ ; 
+				envp[slot_no] = safemalloc(strlen( ASFONT_PATH_ENVVAR ) + 1 + strlen(Environment->font_path)+1) ;
+				sprintf( envp[slot_no], "%s=%s", ASFONT_PATH_ENVVAR, Environment->font_path );
+			}
+		}	 
+			
         len = strlen((char*)cmd);
         if( pass_args )
         {
@@ -930,7 +1282,7 @@ LOCAL_DEBUG_OUT( "len = %d, cmdl = \"%s\" strlen = %d", len, cmdl, strlen(cmdl)
 		}	
 #endif			   
      
-        LOCAL_DEBUG_OUT("execl(\"%s\")", cmdl );
+        LOCAL_DEBUG_OUT("execle(\"%s\")", cmdl );
         /* fprintf( stderr, "len=%d: execl(\"%s\")", len, cmdl ); */
 
         /* CYGWIN does not handle close-on-exec gracefully - whave to do it ourselves : */
@@ -942,12 +1294,12 @@ LOCAL_DEBUG_OUT( "len = %d, cmdl = \"%s\" strlen = %d", len, cmdl, strlen(cmdl)
 			char *argv0 ;
 
 	    	if ((shell = getenv("SHELL")) == NULL || *shell == '\0')
-				shell = "/bin/sh";
+				shell = mystrdup("/bin/sh");
 
 			parse_file_name(shell, NULL, &argv0);
 	    	/* argv0 = basename(shell); */
 
-        	execl (shell, argv0 , "-c", cmdl, (char *)0);
+        	execle (shell, argv0 , "-c", cmdl, (char *)0, envp);
 		}
 
         if( screen >= 0 )
