@@ -185,6 +185,7 @@ void update_tabs_desktop();
 void update_tabs_state();
 void register_unswallowed_client( Window client );
 Bool check_unswallowed_client( Window client ); 
+void delete_tab( int index );
 
 /* above function may also return : */
 #define BANNER_TAB_INDEX -1		   
@@ -291,7 +292,6 @@ for( i = 1 ; i< argc ; ++i)
 	signal (SIGTERM, DeadPipe);
     signal (SIGKILL, DeadPipe);
     
-    balloon_init (False);
     Config = CreateWinTabsConfig ();
 
     LoadBaseConfig ( GetBaseOptions);
@@ -366,8 +366,24 @@ LOCAL_DEBUG_OUT( "DeadPipe%s", "" );
     
 		ASSync(False );
     }	  
-	if( WinTabsState.main_canvas )
-    	destroy_ascanvas( &WinTabsState.main_canvas );
+	
+	while( PVECTOR_USED( WinTabsState.tabs ) ) 
+		delete_tab(0);		
+	destroy_string( &(WinTabsState.banner.name) );
+   	destroy_astbar( &(WinTabsState.banner.bar) );
+   	destroy_ascanvas( &(WinTabsState.banner.client_canvas) );
+   	destroy_ascanvas( &(WinTabsState.banner.frame_canvas) );
+
+	destroy_asvector( &WinTabsState.tabs );
+   	destroy_ascanvas( &WinTabsState.tabs_canvas );
+   	destroy_ascanvas( &WinTabsState.main_canvas );
+
+	destroy_astbar_props( &(WinTabsState.tbar_props) );
+	free_button_resources( &WinTabsState.close_button );
+	free_button_resources( &WinTabsState.unswallow_button );
+	free_button_resources( &WinTabsState.menu_button );
+
+
     if( WinTabsState.main_window )
         XDestroyWindow( dpy, WinTabsState.main_window );
 	ASSync(False );
@@ -393,6 +409,13 @@ void
 retrieve_wintabs_astbar_props()
 {
 	destroy_astbar_props( &(WinTabsState.tbar_props) );
+
+	free_button_resources( &WinTabsState.close_button );
+	free_button_resources( &WinTabsState.unswallow_button );
+	free_button_resources( &WinTabsState.menu_button );
+	memset(&WinTabsState.close_button, 0x00, sizeof(MyButton));
+	memset(&WinTabsState.unswallow_button, 0x00, sizeof(MyButton));
+	memset(&WinTabsState.menu_button, 0x00, sizeof(MyButton));
 	
 	WinTabsState.tbar_props = get_astbar_props(Scr.wmprops );
 	button_from_astbar_props( WinTabsState.tbar_props, &WinTabsState.close_button, 		C_CloseButton, 		_AS_BUTTON_CLOSE, _AS_BUTTON_CLOSE_PRESSED );
@@ -479,7 +502,8 @@ LOCAL_DEBUG_OUT( "exclude_pattern = \"%s\"", Config->exclude_pattern );
     if( get_flags(Config->geometry.flags, YNegative) )
         Config->anchor_y += Scr.MyDisplayHeight ;
 
-    retrieve_wintabs_astbar_props();
+	if( WinTabsState.tbar_props == NULL ) 
+	    retrieve_wintabs_astbar_props();
 	mystyle_get_property (Scr.wmprops);
 
     Scr.Look.MSWindow[BACK_UNFOCUSED] = mystyle_find( Config->unfocused_style );
@@ -488,11 +512,19 @@ LOCAL_DEBUG_OUT( "exclude_pattern = \"%s\"", Config->exclude_pattern );
 
     for( i = 0 ; i < BACK_STYLES ; ++i )
     {
-        static char *default_window_style_name[BACK_STYLES] ={"focused_window_style","unfocused_window_style","sticky_window_style", NULL};
+        static char *default_window_style_name[BACK_STYLES] ={"focused_window_style","unfocused_window_style","sticky_window_style", NULL, NULL};
         if( Scr.Look.MSWindow[i] == NULL )
             Scr.Look.MSWindow[i] = mystyle_find( default_window_style_name[i] );
-        if( Scr.Look.MSWindow[i] == NULL )
+        if( Scr.Look.MSWindow[i] == NULL && i != BACK_URGENT )
             Scr.Look.MSWindow[i] = mystyle_find_or_default( default_style );
+		if( Scr.Look.MSWindow[i] )
+		{
+			if( Scr.Look.MSWindow[i]->texture_type == TEXTURE_SHAPED_PIXMAP )
+				Scr.Look.MSWindow[i]->texture_type = TEXTURE_PIXMAP ;
+			else if( Scr.Look.MSWindow[i]->texture_type == TEXTURE_SHAPED_SCALED_PIXMAP )
+				Scr.Look.MSWindow[i]->texture_type = TEXTURE_SCALED_PIXMAP ;
+			
+		}
     }
     free( default_style );
 
@@ -501,7 +533,7 @@ LOCAL_DEBUG_OUT( "exclude_pattern = \"%s\"", Config->exclude_pattern );
     Print_balloonConfig ( Config->balloon_conf );
 #endif
 	
-	balloon_config2look( &(Scr.Look), Config->balloon_conf, "*WinTabsBalloon" );
+	balloon_config2look( &(Scr.Look), NULL, Config->balloon_conf, "*WinTabsBalloon" );
     set_balloon_look( Scr.Look.balloon_look );
 
 }
@@ -845,7 +877,6 @@ DispatchEvent (ASEvent * event)
 			}
 	        break;
 	    case PropertyNotify:
-			LOCAL_DEBUG_OUT( "property %s(%lX), _XROOTPMAP_ID = %lX, event->w = %lX, root = %lX", XGetAtomName(dpy, event->x.xproperty.atom), event->x.xproperty.atom, _XROOTPMAP_ID, event->w, Scr.Root );
 			if( event->w == Scr.Root || event->w == Scr.wmprops->selection_window ) 
 			{
 				if( event->w == Scr.Root && event->x.xproperty.atom == _XA_WIN_SUPPORTING_WM_CHECK )
@@ -1040,6 +1071,7 @@ make_wintabs_window()
     unsigned int height = max(Config->geometry.height,1);
     XSetWindowAttributes attributes;
 
+	memset( &extwm_hints, 0x00, sizeof(extwm_hints));
     attributes.background_pixmap = ParentRelative;
     switch( Config->gravity )
 	{
@@ -1071,7 +1103,9 @@ make_wintabs_window()
     shints.win_gravity = Config->gravity ;
 
 	extwm_hints.pid = getpid();
-    extwm_hints.flags = EXTWM_PID|EXTWM_TypeASModule|EXTWM_TypeNormal ;
+    extwm_hints.flags = EXTWM_PID|EXTWM_TypeSet ;
+	extwm_hints.type_flags = EXTWM_TypeASModule|EXTWM_TypeNormal ;
+	
 
 	set_client_hints( w, NULL, &shints, AS_DoesWmDeleteWindow, &extwm_hints );
 	set_client_cmd (w);
@@ -1602,7 +1636,7 @@ do_swallow_window( ASWindowData *wd )
 	set_client_desktop (wd->client, WinTabsState.my_desktop );	 
 
 #if 0   /* TODO : implement support for icons : */
-    if( get_flags( wd->flags, AS_ClientIcon ) && !get_flags( wd->flags, AS_ClientIconPixmap) &&
+    if( get_flags( wd->client_icon_flags, AS_ClientIcon ) && !get_flags( wd->client_icon_flags, AS_ClientIconPixmap) &&
 		wd->icon != None )
     {
         ASCanvas *ic = create_ascanvas_container( wd->icon );
