@@ -368,6 +368,7 @@ LOCAL_DEBUG_OUT( "--DESTR Client(%lx(%s))->CLIENT->canvas(%p)->window(%lx)", asw
         if( (asw->status == NULL || !get_flags(asw->status->flags, AS_Dead)) && 
 			get_parent_window( w ) == asw->frame )
         {
+			LOCAL_DEBUG_OUT( "reparenting client window %lX", w );
             if( get_flags( AfterStepState, ASS_Shutdown ) )
                 quietly_reparent_window( w, Scr.Root, xwc.x, xwc.y, AS_CLIENT_EVENT_MASK );
             else
@@ -401,7 +402,7 @@ check_icon_canvas( ASWindow *asw, Bool required )
             attributes.border_pixel = Scr.asv->black_pixel;
             attributes.cursor = Scr.Feel.cursors[ASCUR_Default];
 
-            if ((ASWIN_HFLAGS(asw, AS_ClientIcon|AS_ClientIconPixmap) != AS_ClientIcon) ||
+            if ((get_flags(asw->hints->client_icon_flags, AS_ClientIcon|AS_ClientIconPixmap) != AS_ClientIcon) ||
                  asw->hints == NULL || asw->hints->icon.window == None ||
                  !get_flags(Scr.Feel.flags, KeepIconWindows))
             { /* create windows */
@@ -957,7 +958,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
 	{
 		if( old_hints )
 		{
-			icon_image_changed = (ASWIN_HFLAGS( asw, AS_ClientIcon|AS_ClientIconPixmap ) != get_flags(old_hints->flags, AS_ClientIcon|AS_ClientIconPixmap));
+			icon_image_changed = (get_flags(asw->hints->client_icon_flags, AS_ClientIcon|AS_ClientIconPixmap ) != get_flags(old_hints->client_icon_flags, AS_ClientIcon|AS_ClientIconPixmap));
 			if( !icon_image_changed )
         		icon_image_changed = (asw->hints->icon.pixmap != old_hints->icon.pixmap );
 			if( !icon_image_changed )
@@ -1234,13 +1235,19 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
       		    {
           		    char *str = list_functions_by_context (C_TButton0<<i, asw->hints);
               		LOCAL_DEBUG_OUT( "balloon text will be \"%s\"", str?str:"none" );
-	                set_astbar_balloon( asw->tbar, C_TButton0<<i, str, AS_Text_ASCII );
+	                set_astbar_balloon2( asw->tbar, TitlebarBalloons, C_TButton0<<i, str, AS_Text_ASCII );
   		            if( str )
       		            free( str );
           		}
       		}
 		}
 	}
+	
+	if( ASWIN_HFLAGS( asw, AS_WindowOpacity ) )
+	{
+		set_32bit_property (asw->frame, _XA_NET_WM_WINDOW_OPACITY, XA_CARDINAL, asw->hints->window_opacity );
+	}	 
+
 
     /* we also need to setup label, unfocused/sticky style and tbar sizes -
      * it all is done when we change windows state, or move/resize it */
@@ -1251,14 +1258,18 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
 		/* we need to map all the possibly created subwindows if window is not iconic */
 		if( !ASWIN_GET_FLAGS(asw, AS_Iconic ) )
 		{
-			LOCAL_DEBUG_OUT("mapping frame subwindows for client %lX, frame canvas = %p", asw->w, asw->frame_canvas );
-			XMapSubwindows(dpy, asw->frame);
+			for( i = 0 ; i < FRAME_SIDES ; ++i )
+				map_canvas_window( asw->frame_sides[i], False );
+
+/*			LOCAL_DEBUG_OUT("mapping frame subwindows for client %lX, frame canvas = %p", asw->w, asw->frame_canvas );
+			XMapSubwindows(dpy, asw->frame); */
 		}else
 		{
 			map_canvas_window( asw->icon_canvas, False );
 			map_canvas_window( asw->icon_title_canvas, False );
 		}
 	}
+
 	return	status_changed ;
 }
 
