@@ -43,6 +43,7 @@
 #define STOP_LONG_DRAW_OPERATION    ungrab_server()
 
 
+
 static ASMenu *ASTopmostMenu = NULL;
 
 ASHints *make_menu_hints( ASMenu *menu );
@@ -86,7 +87,9 @@ free_asmenu_item( ASMenuItem *item )
 {
     if( item->bar )
         destroy_astbar(&(item->bar) );
-    item->submenu = NULL ;
+	if( item->icon )
+	    safe_asimage_destroy( item->icon );
+	free_func_data (&(item->fdata));
 }
 
 void destroy_asmenu(ASMenu **pmenu);
@@ -103,7 +106,7 @@ LOCAL_DEBUG_CALLER_OUT( "top(%p)->supermenu(%p)->menu(%p)->submenu(%p)", ASTopmo
         {
             /* cannot use Destroy directly - must go through the normal channel: */
             unmap_canvas_window( menu->submenu->main_canvas );
-            ASWIN_CLEAR_FLAGS(menu->submenu->owner, AS_Visible);
+            ASWIN_SET_FLAGS(menu->submenu->owner, AS_Hidden);
             ASWIN_SET_FLAGS(menu->submenu->owner, AS_UnMapPending);
 			menu->submenu = NULL ;
         }else
@@ -134,7 +137,7 @@ LOCAL_DEBUG_CALLER_OUT( "top(%p)->supermenu(%p)->menu(%p)->submenu(%p)", ASTopmo
             if( menu->owner )
             {
                 ASWIN_SET_FLAGS(menu->owner, AS_Dead );
-                ASWIN_CLEAR_FLAGS(menu->owner, AS_Visible);
+                ASWIN_SET_FLAGS(menu->owner, AS_Hidden);
                 ASWIN_SET_FLAGS(menu->owner, AS_UnMapPending);
             }
             destroy_registered_window( w );
@@ -157,6 +160,9 @@ LOCAL_DEBUG_CALLER_OUT( "top(%p)->supermenu(%p)->menu(%p)->submenu(%p)", ASTopmo
                 free( menu->name );
             if( menu->title )
                 free( menu->title );
+
+			destroy_asballoon( &(menu->comment_balloon) );
+
             menu->magic = 0 ;
             free( menu );
             *pmenu = NULL ;
@@ -183,8 +189,8 @@ LOCAL_DEBUG_CALLER_OUT( "top(%p)->supermenu(%p)->menu(%p)->submenu(%p)", ASTopmo
             {
                 /* cannot use Destroy directly - must go through the normal channel: */
                 unmap_canvas_window( menu->main_canvas );
-                ASWIN_CLEAR_FLAGS(menu->owner, AS_Visible);
-                ASWIN_SET_FLAGS(menu->owner, AS_UnMapPending);
+                ASWIN_SET_FLAGS(menu->owner, AS_Hidden );
+                ASWIN_SET_FLAGS(menu->owner, AS_UnMapPending );
             }else
                 destroy_asmenu( &(menu));
             *pmenu = NULL ;
@@ -192,6 +198,8 @@ LOCAL_DEBUG_CALLER_OUT( "top(%p)->supermenu(%p)->menu(%p)->submenu(%p)", ASTopmo
     }
 }
 
+#define FDataPopupName(fdata)   ((fdata).text)
+
 static void
 set_asmenu_item_data( ASMenuItem *item, MenuDataItem *mdi )
 {
@@ -200,8 +208,9 @@ set_asmenu_item_data( ASMenuItem *item, MenuDataItem *mdi )
     item->source = mdi ;
 
     if( item->bar == NULL )
+	{
         item->bar = create_astbar();
-	else
+	}else
 		delete_astbar_tile( item->bar, -1 );  /* delete all tiles */
 
 
@@ -211,17 +220,25 @@ set_asmenu_item_data( ASMenuItem *item, MenuDataItem *mdi )
         item->icon = NULL ;
     }
 	/* we can only use images that are reference counted */
-    if( mdi->minipixmap_image && mdi->minipixmap_image->imageman != NULL )
+    if( mdi->minipixmap_image )
         icon_im = mdi->minipixmap_image ;
     else if( mdi->minipixmap )
         icon_im = GetASImageFromFile( mdi->minipixmap );
+	else if( mdi->fdata->func == F_CHANGE_BACKGROUND_FOREIGN ) 
+		icon_im = GetASImageFromFile( mdi->fdata->text );
+
     if( icon_im )
 	{
-		item->icon = check_scale_menu_pmap( icon_im );
+		item->icon = check_scale_menu_pmap( icon_im, mdi->flags );
 		if( item->icon != icon_im && icon_im != mdi->minipixmap_image )
 			safe_asimage_destroy( icon_im );
-		if( item->icon == mdi->minipixmap_image )
-            item->icon = dup_asimage( item->icon );
+		if( item->icon && item->icon == mdi->minipixmap_image )
+		{	
+			if( item->icon->imageman != NULL )
+            	item->icon = dup_asimage( item->icon );
+			else
+		    	item->icon = clone_asimage( item->icon, 0xFFFFFFFF );
+		}
     }
 LOCAL_DEBUG_OUT( "item(\"%s\")->minipixmap(\"%s\")->icon(%p)", mdi->item?mdi->item:"NULL", mdi->minipixmap?mdi->minipixmap:"NULL", item->icon );
 
@@ -239,21 +256,30 @@ LOCAL_DEBUG_OUT( "item(\"%s\")->minipixmap(\"%s\")->icon(%p)", mdi->item?mdi->it
     /* optional menu items : */
     /* add label */
     if( mdi->item )
+	{	
         add_astbar_label( item->bar, 3, 0, 0, ALIGN_LEFT|ALIGN_VCENTER, 0, 0, mdi->item,
 		mdi->fdata->name_encoding );
+		item->first_sym = mdi->item[0] ;
+	}
     /* add hotkey */
     if( mdi->item2 )
         add_astbar_label( item->bar, 4, 0, 0, ALIGN_RIGHT|ALIGN_VCENTER, 0, 0, mdi->item2, mdi->fdata->name_encoding );
 
     item->flags = 0 ;
-    if( mdi->fdata->func == F_POPUP )
+
+	if( IsDynamicPopup(mdi->fdata->func) )
+	{	
+		set_flags( item->flags, AS_MenuItemHasSubmenu );
+	}else if( mdi->fdata->func == F_POPUP )
     {
-        if( (item->submenu = FindPopup (mdi->fdata->text, True)) == NULL )
+		MenuData *submenu = FindPopup (FDataPopupName(*(mdi->fdata)), True);
+        if( submenu == NULL )
             set_flags( item->flags, AS_MenuItemDisabled );
 		else
 		{
-			MenuDataItem *smdi = item->submenu->first ;
+			MenuDataItem *smdi = submenu->first ;
 
+			set_flags( item->flags, AS_MenuItemHasSubmenu );
 			while( smdi ) 
 			{
 				if( smdi->fdata->func != F_TITLE )
@@ -266,7 +292,7 @@ LOCAL_DEBUG_OUT( "item(\"%s\")->minipixmap(\"%s\")->icon(%p)", mdi->item?mdi->it
     }else if( get_flags( mdi->flags, MD_Disabled ) || mdi->fdata->func == F_NOP )
         set_flags( item->flags, AS_MenuItemDisabled );
 
-    item->fdata = mdi->fdata ;
+	dup_func_data (&(item->fdata), mdi->fdata);
 }
 
 static Bool
@@ -279,6 +305,12 @@ LOCAL_DEBUG_OUT( "item.bar(%p)->look(%p)", item->bar, look );
     if( item->bar == NULL )
         return False;
 
+	if( get_flags( look->flags, TxtrMenuItmInd ) )
+		clear_flags( item->bar->state, BAR_FLAGS_CROP_BACK );
+	else
+		set_flags( item->bar->state, BAR_FLAGS_CROP_UNFOCUSED_BACK );
+
+
     item->bar->h_spacing = DEFAULT_MENU_SPACING ;
     item->bar->h_border = DEFAULT_MENU_ITEM_HBORDER ;
     item->bar->v_border = DEFAULT_MENU_ITEM_VBORDER ;
@@ -315,7 +347,7 @@ LOCAL_DEBUG_OUT( "item.bar(%p)->look(%p)", item->bar, look );
     /* delete tile from Popup arrow cell : */
     delete_astbar_tile( item->bar, MI_POPUP_IDX );
     /* now readd it as proper type : */
-    if( look->MenuArrow && item->fdata->func == F_POPUP )
+    if( look->MenuArrow && get_flags( item->flags, AS_MenuItemHasSubmenu ) )
         add_astbar_icon( item->bar, 7, 0, 0, ALIGN_VCENTER, look->MenuArrow->image );
     else
         add_astbar_spacer( item->bar, 7, 0, 0, NO_ALIGN, arrow_space, 1 );
@@ -378,6 +410,7 @@ render_asmenu_bars( ASMenu *menu, Bool force )
     Bool rendered = False;
     if( menu->main_canvas->height > 1 && menu->main_canvas->width > 1 )
     {
+		ASImage *cache = NULL ; 
         START_LONG_DRAW_OPERATION;
         for( i = 0 ; i < menu->items_num ; ++i )
         {
@@ -396,7 +429,7 @@ render_asmenu_bars( ASMenu *menu, Bool force )
                 bar->win_y + (int)(bar->height) > 0 &&
                 (int)(bar->win_y) > prev_y )
             {
-                if( render_astbar( bar, menu->main_canvas ) )
+                if( render_astbar_cached_back( bar, menu->main_canvas, &cache, NULL ) )
                     rendered = True ;
             }
             prev_y = bar->win_y ;
@@ -404,10 +437,10 @@ render_asmenu_bars( ASMenu *menu, Bool force )
 		if( menu->visible_items_num < menu->items_num )
 		{
 			if( force || DoesBarNeedsRendering(menu->scroll_up_bar))
-            	if( render_astbar( menu->scroll_up_bar, menu->main_canvas ) )
+            	if( render_astbar_cached_back( menu->scroll_up_bar, menu->main_canvas, &cache, NULL ) )
                 	rendered = True ;
 			if( force || DoesBarNeedsRendering(menu->scroll_down_bar))
-            	if( render_astbar( menu->scroll_down_bar, menu->main_canvas ) )
+            	if( render_astbar_cached_back( menu->scroll_down_bar, menu->main_canvas, &cache, NULL ) )
                 	rendered = True ;
 		}
         STOP_LONG_DRAW_OPERATION;
@@ -416,8 +449,10 @@ render_asmenu_bars( ASMenu *menu, Bool force )
         {
             update_canvas_display( menu->main_canvas );
             ASSync(False);
-			menu->rendered = True ;
+			set_flags(	menu->state, AS_MenuRendered);
         }
+		if( cache ) 
+			safe_asimage_destroy( cache );
     }
 }
 
@@ -425,9 +460,10 @@ render_asmenu_bars( ASMenu *menu, Bool force )
 /* midium level ASMenu functionality :                                      */
 /*************************************************************************/
 static unsigned int
-extract_recent_subitems( MenuData *md, MenuDataItem **subitems, unsigned int max_subitems )
+extract_recent_subitems( char *submenu_name, MenuDataItem **subitems, unsigned int max_subitems )
 {
-    int used = 0 ;
+	MenuData *md = FindPopup (submenu_name, True);
+	int used = 0 ;
     MenuDataItem *smdi = md->first ;
     while( smdi != NULL )
     {
@@ -454,7 +490,7 @@ extract_recent_subitems( MenuData *md, MenuDataItem **subitems, unsigned int max
 }
 
 void
-set_asmenu_data( ASMenu *menu, MenuData *md, Bool first_time )
+set_asmenu_data( ASMenu *menu, MenuData *md, Bool first_time, Bool show_unavailable, int recent_items )
 {
     int items_num = md->items_num ;
     int i = 0 ;
@@ -478,9 +514,12 @@ set_asmenu_data( ASMenu *menu, MenuData *md, Bool first_time )
     if( items_num > 0 )
     {
         MenuDataItem *mdi = md->first ;
+		if( md->recent_items >= 0 )
+			recent_items = md->recent_items ;
+        if( recent_items > 0 )
+            subitems = safecalloc( recent_items, sizeof(MenuDataItem*));
 
-        if( md->recent_items > 0 )
-            subitems = safecalloc( md->recent_items, sizeof(MenuDataItem*));
+		LOCAL_DEBUG_OUT( "show_unavailable = %d", show_unavailable );
 
         for(mdi = md->first; real_items_num < items_num && mdi != NULL ; mdi = mdi->next )
             if( mdi->fdata->func == F_TITLE && menu->title == NULL )
@@ -489,12 +528,27 @@ set_asmenu_data( ASMenu *menu, MenuData *md, Bool first_time )
             }else
             {
                 ASMenuItem *item = &(menu->items[real_items_num]);
-                set_asmenu_item_data( item, mdi );
+				
+				if( !show_unavailable ) 
+				{
+					if( get_flags( mdi->flags, MD_Disabled ) )
+						continue;
+					if( mdi->fdata->func == F_NOP  )
+					{	
+						int len = mdi->item?strlen(mdi->item):0 ;
+						if(  mdi->item2 )
+							len += strlen(mdi->item2);
+						if( len > 0 )
+							continue;
+					}
+				}
 
+                set_asmenu_item_data( item, mdi );
                 ++real_items_num;
-                if( mdi->fdata->func == F_POPUP && item->submenu != NULL && subitems != NULL )
+
+                if( item->fdata.func == F_POPUP && !get_flags( item->flags, AS_MenuItemDisabled ) && subitems != NULL )
                 {
-                    int used = extract_recent_subitems( item->submenu, subitems, md->recent_items );
+                    int used = extract_recent_subitems( FDataPopupName(item->fdata), subitems, recent_items );
                     if( used > 0 )
                     {
                         items_num += used ;
@@ -547,6 +601,16 @@ set_asmenu_data( ASMenu *menu, MenuData *md, Bool first_time )
 	}else
 		menu->selected_item = -1 ;
     menu->pressed_item = -1;
+	
+	if( md->comment != NULL ) 
+	{
+		if( menu->comment_balloon == NULL )
+			menu->comment_balloon = create_asballoon_with_text_for_state ( MenuBalloons, NULL, md->comment, 0);
+		else
+			balloon_set_text (menu->comment_balloon, md->comment, 0);
+		clear_flags( menu->state, AS_MenuBalloonShown);
+	}else if( menu->comment_balloon )
+		destroy_asballoon( &(menu->comment_balloon) );
 }
 
 void
@@ -611,7 +675,10 @@ set_asmenu_look( ASMenu *menu, MyLook *look )
         if( (bar= menu->items[i].bar) != NULL )
         {
             width = calculate_astbar_width( bar );
-            height = calculate_astbar_height( bar );
+			if( get_flags( menu->items[i].flags, AS_MenuItemSubitem ) )
+				height = max_height;
+			else
+            	height = calculate_astbar_height( bar );
 LOCAL_DEBUG_OUT( "i(%d)->bar(%p)->size(%ux%u)", i, bar, width, height );
             if( width > max_width )
                 max_width = width ;
@@ -690,9 +757,11 @@ set_menu_item_used( ASMenu *menu, MenuDataItem *mdi )
 void set_asmenu_scroll_position( ASMenu *menu, int pos );
 
 void
-select_menu_item( ASMenu *menu, int selection )
+select_menu_item( ASMenu *menu, int selection, Bool render )
 {
+	Bool needs_scrolling ;
 LOCAL_DEBUG_CALLER_OUT( "%p,%d", menu, selection );
+	
     if( AS_ASSERT(menu) || menu->items_num == 0 )
         return;
     if( selection < 0 )
@@ -700,20 +769,22 @@ LOCAL_DEBUG_CALLER_OUT( "%p,%d", menu, selection );
     else if( selection >= (int)menu->items_num )
         selection = menu->items_num - 1 ;
 
+	needs_scrolling = ( selection < menu->top_item || selection >= menu->top_item + menu->visible_items_num ) ;
     if( selection != menu->selected_item )
     {
         close_asmenu_submenu( menu );
 		if( menu->selected_item >= 0 )
-        	set_astbar_focused( menu->items[menu->selected_item].bar, menu->main_canvas, False );
+        	set_astbar_focused( menu->items[menu->selected_item].bar, NULL/*needs_scrolling?NULL:menu->main_canvas*/, False );
     }
-    set_astbar_focused( menu->items[selection].bar, menu->main_canvas, True );
+    set_astbar_focused( menu->items[selection].bar, NULL/*needs_scrolling?NULL:menu->main_canvas*/, True );
     menu->selected_item = selection ;
 
     if( selection < menu->top_item )
-        set_asmenu_scroll_position( menu, MAX(selection-1, 0) );
+        set_asmenu_scroll_position( menu, MAX(selection, 0) );
     else if( selection >= menu->top_item + menu->visible_items_num )
         set_asmenu_scroll_position( menu, (selection-menu->visible_items_num)+1);
-    render_asmenu_bars(menu, False);
+	else if( render )
+	    render_asmenu_bars(menu, False);
 }
 
 void
@@ -764,41 +835,63 @@ LOCAL_DEBUG_OUT("adj_pos(%d)->curr_y(%d)->items_num(%d)->vis_items_num(%d)->sel_
 	menu->top_item = first_item ;
 
 	if( menu->selected_item < menu->top_item )
-        select_menu_item( menu, menu->top_item );
+        select_menu_item( menu, menu->top_item, False );
     else if( menu->selected_item >= 0 )
 	{
 		if( menu->visible_items_num > 0 && menu->selected_item >= (int)menu->top_item + (int)menu->visible_items_num )
-        select_menu_item( menu, menu->top_item + menu->visible_items_num - 1 );
+        select_menu_item( menu, menu->top_item + menu->visible_items_num - 1, False );
 	}
     render_asmenu_bars(menu, False);
 }
 
 static inline void
-run_item_submenu( ASMenu *menu, int item )
+run_item_submenu( ASMenu *menu, int item_no )
 {
-LOCAL_DEBUG_CALLER_OUT( "%p, %d, submenu(%p)", menu, item, menu->items[item].submenu );
-    if( menu->items[item].submenu && !get_flags( menu->items[item].flags, AS_MenuItemDisabled ) )
-    {
-        int x = menu->main_canvas->root_x+(int)menu->main_canvas->bw ;
-        int y ;
-        ASQueryPointerRootXY( &x, &y );
-        x -= 5 ;
+	ASMenuItem *item = &(menu->items[item_no]);
+LOCAL_DEBUG_CALLER_OUT( "%p, %d", menu, item_no );
+    if( !get_flags( item->flags, AS_MenuItemDisabled ) )
+	{
+		MenuData *submenu = NULL ;
+		if( item->fdata.func == F_POPUP )
+			submenu = FindPopup( FDataPopupName(item->fdata), True );
+		else if( item->fdata.func == F_STOPMODULELIST ) 
+			submenu = make_stop_module_menu(  Scr.Feel.winlist_sort_order );
+		else if( item->fdata.func == F_RESTARTMODULELIST ) 
+			submenu = make_restart_module_menu(  Scr.Feel.winlist_sort_order );
+#ifndef NO_WINDOWLIST
+		else if( item->fdata.func == F_WINDOWLIST ) 
+		{	
+			submenu = make_desk_winlist_menu( Scr.Windows, 
+											  (item->fdata.text == NULL) ? 
+											  Scr.CurrentDesk: item->fdata.func_val[0], 
+											  Scr.Feel.winlist_sort_order, False );
+		}
+#endif		
+		if( submenu )
+    	{
+        	int x = menu->main_canvas->root_x+(int)menu->main_canvas->bw ;
+        	int y ;
+        	ASQueryPointerRootXY( &x, &y );
+        	x -= 5 ;
 #if 0
-        if( x  < menu->main_canvas->root_x + (int)(menu->item_width/3) )
-        {
-            int max_dx = Scr.MyDisplayWidth / 40 ;
-            if( menu->main_canvas->root_x + (int)(menu->item_width/3) - x  < max_dx )
-                x = menu->main_canvas->root_x + (int)(menu->item_width/3) ;
-            else
-                x += max_dx ;
-        }
+        	if( x  < menu->main_canvas->root_x + (int)(menu->item_width/3) )
+        	{
+            	int max_dx = Scr.MyDisplayWidth / 40 ;
+            	if( menu->main_canvas->root_x + (int)(menu->item_width/3) - x  < max_dx )
+                	x = menu->main_canvas->root_x + (int)(menu->item_width/3) ;
+            	else
+                	x += max_dx ;
+        	}
 #endif
-        y = menu->main_canvas->root_y+(menu->item_height*(item+1-(int)menu->top_item)) - 10 ;
-/*	if( x > menu->main_canvas->root_x+menu->item_width-5 )
-	    x = menu->main_canvas->root_x+menu->item_width-5 ;
-*/      close_asmenu_submenu( menu );
-        menu->submenu = run_submenu( menu, menu->items[item].submenu, x, y);
-    }
+        	y = menu->main_canvas->root_y+(menu->item_height*(item_no+1-(int)menu->top_item)) - 10 ;
+	/*	if( x > menu->main_canvas->root_x+menu->item_width-5 )
+	    	x = menu->main_canvas->root_x+menu->item_width-5 ;
+	*/      close_asmenu_submenu( menu );
+        	menu->submenu = run_submenu( menu, submenu, x, y);
+			if( item->fdata.func != F_POPUP )
+    			destroy_menu_data( &submenu );
+		}
+	}
 }
 
 void
@@ -824,17 +917,17 @@ LOCAL_DEBUG_CALLER_OUT( "%p,%d", menu, pressed );
 			if( keyboard_event ) 
 			{   /* don't press if keyboard selection */
 				if( pressed != menu->selected_item )
-					select_menu_item( menu, pressed );
+					select_menu_item( menu, pressed, False );
 
 			}else if( pressed != menu->selected_item )
             {
                 set_astbar_pressed( menu->items[pressed].bar, NULL, True );/* don't redraw yet */
-                select_menu_item( menu, pressed );  /* this one updates display already */
+                select_menu_item( menu, pressed, False );  /* this one updates display already */
             }else
                 set_astbar_pressed( menu->items[pressed].bar, menu->main_canvas, True );
             set_menu_item_used( menu, menu->items[pressed].source );
 			
-			if( menu->items[pressed].submenu != NULL ) 
+			if( get_flags( menu->items[pressed].flags, AS_MenuItemHasSubmenu) ) 
 			{
 				if( keyboard_event ) 
 				{
@@ -848,8 +941,8 @@ LOCAL_DEBUG_CALLER_OUT( "%p,%d", menu, pressed );
         }
     }
     render_asmenu_bars(menu, False);
-LOCAL_DEBUG_OUT( "pressed(%d)->old_pressed(%d)->focused(%d)", pressed, menu->pressed_item, menu->focused );
-    if( menu->focused )
+LOCAL_DEBUG_OUT( "pressed(%d)->old_pressed(%d)->focused(%d)", pressed, menu->pressed_item, get_flags(menu->state, AS_MenuFocused)?1:0 );
+    if( get_flags(menu->state, AS_MenuFocused) )
 	{	
 		int item_no = keyboard_event?pressed:menu->pressed_item ;
     	
@@ -857,12 +950,11 @@ LOCAL_DEBUG_OUT( "pressed(%d)->old_pressed(%d)->focused(%d)", pressed, menu->pre
 		if( item_no >= 0 && (pressed < 0 || keyboard_event))   
     	{
         	ASMenuItem *item = &(menu->items[item_no]);
-        	if( !get_flags(item->flags, AS_MenuItemDisabled) &&
-            	 item->fdata->func != F_POPUP )
+        	if( get_flags(item->flags, AS_MenuItemDisabled|AS_MenuItemHasSubmenu) == 0 )
         	{
             	set_menu_item_used( menu, item->source );
-            	ExecuteFunctionForClient( item->fdata, menu->client_window );
-            	if( !menu->pinned )
+            	ExecuteFunctionForClient( &(item->fdata), menu->client_window );
+            	if( !get_flags(menu->state, AS_MenuPinned) )
                 	close_asmenu( &ASTopmostMenu );
         	}
     	}
@@ -934,7 +1026,10 @@ LOCAL_DEBUG_CALLER_OUT( "%p, %p, %d", asiw, data, focused );
     {
         /* TODO : hilite/unhilite selected item, and
          * withdraw non-pinned menu if it has no submenus */
-        menu->focused = focused ;
+		if( focused ) 
+		 	set_flags(menu->state, AS_MenuFocused);
+		else
+			clear_flags(menu->state, AS_MenuFocused);
     }
 }
 
@@ -983,6 +1078,70 @@ LOCAL_DEBUG_OUT( "pointer(%d,%d)", px, py );
     }
 }
 
+static void 
+handle_menu_selection( ASMenu *menu, int button_state, int px, int selection, Bool render)
+{
+    if( selection != menu->selected_item )
+    {
+        if( button_state & (Button1Mask|Button2Mask|Button3Mask) )
+            press_menu_item( menu, selection, False );
+        else
+            select_menu_item( menu, selection, render );
+    }
+    if( px > menu->main_canvas->width - ( menu->arrow_space + DEFAULT_MENU_SPACING ) &&
+        menu->submenu == NULL )
+        run_item_submenu( menu, selection );
+
+}
+
+void
+on_menu_scroll_event( ASInternalWindow *asiw, ASEvent *event )
+{
+    ASMenu   *menu = (ASMenu*)(asiw->data) ;
+    if( menu != NULL && menu->magic == MAGIC_ASMENU && event)
+    {
+		if(  menu->items_num >= menu->visible_items_num )
+		{
+			XButtonEvent *xbtn = &(event->x.xbutton);
+			int selection = menu->selected_item ; 
+			int top = menu->top_item ;
+			int px_offset = menu->main_canvas->root_x+(int)menu->main_canvas->bw ;
+			int px, py ;
+			Bool needs_scrolling ;
+			
+			ASQueryPointerRootXY( &px, &py );
+			if( xbtn->button == Button4 )		
+			{
+				if( selection > 0 ) 
+				{
+					--selection ;
+					py -= menu->item_height ;
+				}
+				if( selection < menu->top_item )
+					--top ;
+			}else
+			{
+				if( selection+1 < menu->items_num ) 
+				{
+					++selection ;
+					py += menu->item_height ;
+				}
+				if( selection > menu->top_item + menu->visible_items_num - 1 )
+					++top ;
+			}
+			needs_scrolling = (top != menu->top_item);
+			if( selection != menu->selected_item )
+			{
+				handle_menu_selection( menu, xbtn->state, px-px_offset, selection, !needs_scrolling);
+				if( !needs_scrolling ) 
+					XWarpPointer (dpy, None, Scr.Root, 0, 0, 0, 0, px, py);
+			}
+//			if( needs_scrolling ) 
+//				set_asmenu_scroll_position( menu, top );
+		}
+	}
+}
+
 /* Motion notify : */
 void
 on_menu_pointer_event( ASInternalWindow *asiw, ASEvent *event )
@@ -998,9 +1157,15 @@ on_menu_pointer_event( ASInternalWindow *asiw, ASEvent *event )
 		XEvent tmp_e ;
 		if( ASCheckTypedWindowEvent(canvas->w,LeaveNotify,&tmp_e) )
 		{
+            withdraw_balloon( menu->comment_balloon );
 			XPutBackEvent( dpy, &tmp_e );
 			return ;    /* pointer has moved into other window - ignore this event! */
 		}
+		if( menu->comment_balloon && !get_flags( menu->state, AS_MenuBalloonShown)) 
+		{
+   			set_flags( menu->state, AS_MenuBalloonShown);
+		    display_balloon( menu->comment_balloon );
+		}
 		/* must get current pointer position as MotionNotify events
 		   tend to accumulate while we are drawing and we start getting
 		   late with menu selection, creating an illusion of slowness */
@@ -1046,16 +1211,7 @@ on_menu_pointer_event( ASInternalWindow *asiw, ASEvent *event )
 				selection = py/menu->item_height ;
 			selection += menu->top_item ;
 
-            if( selection != menu->selected_item )
-            {
-                if( xmev->state & ButtonAnyMask )
-                    press_menu_item( menu, selection, False );
-                else
-                    select_menu_item( menu, selection );
-            }
-            if( px > canvas->width - ( menu->arrow_space + DEFAULT_MENU_SPACING ) &&
-                menu->submenu == NULL )
-                run_item_submenu( menu, selection );
+			handle_menu_selection( menu, xmev->state, px, selection, True);
         }
     }
 }
@@ -1102,17 +1258,14 @@ on_menu_keyboard_event( ASInternalWindow *asiw, ASEvent *event )
 			LOCAL_DEBUG_OUT( "processing keysym [%c]", (char)keysym );
 			/* Search menu for matching hotkey */
 			for( i = 0 ; i < menu->items_num ; i++ )
-				if( menu->items[i].fdata->hotkey == keysym )
+				if( menu->items[i].fdata.hotkey == keysym )
 				{
 					press_menu_item( menu, i, True );
 					return ;
 				}
 			for( i = 0 ; i < menu->items_num ; i++ )
 			{
-				char *n = menu->items[i].source->item ;
-		
-				LOCAL_DEBUG_OUT( "name = %s", n?n:"(null)" );	  
-				if( n != NULL && n[0] == keysym )
+				if( menu->items[i].first_sym == keysym )
 				{
 					press_menu_item( menu, i, True );
 					return ;
@@ -1130,22 +1283,22 @@ on_menu_keyboard_event( ASInternalWindow *asiw, ASEvent *event )
 				case XK_k:
 				case XK_p:
 					if( menu->selected_item > 0 )
-	                	select_menu_item( menu, menu->selected_item-1 );
+	                	select_menu_item( menu, menu->selected_item-1, True );
                     break;
 				case XK_Tab :
 				case XK_Down:
 				case XK_n:
 				case XK_j:
 					if( menu->selected_item < menu->items_num )
-	                	select_menu_item( menu, menu->selected_item+1 );
+	                	select_menu_item( menu, menu->selected_item+1, True );
                     break;
 				case XK_Right:
 				case XK_Return :
 				case XK_space :
 					if( menu->selected_item	>= 0 ) 
 					{	
-						if( menu->items[menu->selected_item].submenu || keysym != XK_Right ) 
-		 				press_menu_item( menu, menu->selected_item, True );		
+						if( get_flags( menu->items[menu->selected_item].flags, AS_MenuItemHasSubmenu) || keysym != XK_Right ) 
+		 					press_menu_item( menu, menu->selected_item, True );		
 					}
 			}	 
 		}	
@@ -1162,7 +1315,9 @@ on_menu_look_feel_changed( ASInternalWindow *asiw, ASFeel *feel, MyLook *look, A
     {
 		ASHints *hints ;
 		unsigned int tbar_width ;
+		MenuData *md ;
 
+   /*      if( get_flags( what, FEEL_CONFIG ) )       */
         if( menu->items )
         {
             register int i = menu->items_num;
@@ -1173,16 +1328,13 @@ on_menu_look_feel_changed( ASInternalWindow *asiw, ASFeel *feel, MyLook *look, A
 			menu->items_num = 0 ;
         }
 
-        if( get_flags( what, FEEL_CONFIG ) )
+        md = FindPopup (menu->name, False);
+        if( md == NULL )
         {
-            MenuData *md = FindPopup (menu->name, False);
-            if( md == NULL )
-            {
-                menu_destroy( asiw );
-                return ;
-            }
-            set_asmenu_data( menu, md, False );
+        	menu_destroy( asiw );
+            return ;
         }
+        set_asmenu_data( menu, md, False, get_flags(look->flags, MenuShowUnavailable), feel->recent_submenu_items );
         set_asmenu_look( menu, look );
 		hints = make_menu_hints( menu );
 	    estimate_titlebar_size( hints, &tbar_width, NULL );
@@ -1307,6 +1459,7 @@ show_asmenu( ASMenu *menu, int x, int y )
     asiw->on_hilite_changed = on_menu_hilite_changed ;
     asiw->on_pressure_changed = on_menu_pressure_changed;
     asiw->on_pointer_event = on_menu_pointer_event;
+	asiw->on_scroll_event = on_menu_scroll_event;
     asiw->on_keyboard_event = on_menu_keyboard_event;
     asiw->on_look_feel_changed = on_menu_look_feel_changed;
 	asiw->on_root_background_changed = on_menu_root_background_changed ;
@@ -1425,9 +1578,9 @@ run_submenu( ASMenu *supermenu, MenuData *md, int x, int y )
     ASMenu *menu = NULL ;
     if( md )
     {
-        menu = create_asmenu(md->name);
-		menu->rendered = False ;
-        set_asmenu_data( menu, md, True );
+		menu = create_asmenu(md->name);
+		clear_flags( menu->state, AS_MenuRendered);
+		set_asmenu_data( menu, md, True, get_flags(Scr.Look.flags, MenuShowUnavailable), Scr.Feel.recent_submenu_items );
         set_asmenu_look( menu, &Scr.Look );
         /* will set scroll position when ConfigureNotify arrives */
         menu->supermenu = supermenu;
@@ -1491,7 +1644,7 @@ ASMenu *
 find_topmost_transient_menu( ASMenu *menu )
 {
     if( menu && menu->supermenu && menu->supermenu->magic == MAGIC_ASMENU )
-        if( !menu->supermenu->pinned )
+        if( !get_flags(menu->supermenu->state, AS_MenuPinned) )
             return find_topmost_transient_menu( menu->supermenu );
     return menu;
 }
@@ -1513,7 +1666,7 @@ pin_asmenu( ASMenu *menu )
         if( menu_to_close != menu )
             close_asmenu( &menu_to_close );
 
-        menu->pinned = True ;
+		set_flags(menu->state, AS_MenuPinned);
         if( menu->owner )
         {
             clear_flags( menu->owner->hints->function_mask, AS_FuncPinMenu);
@@ -1530,7 +1683,7 @@ Bool
 is_menu_pinnable( ASMenu *menu )
 {
     if( menu && menu->magic == MAGIC_ASMENU )
-        return !(menu->pinned);
+        return !get_flags(menu->state, AS_MenuPinned);
     return False;
 }
 
